/*  Copyright Spencer Hakim 2010. All rights reserved.
 *  License: http://www.gnu.org/licenses/gpl-3.0.txt
 *  This is simple web client. It writes HTTP responses from servers to 'webscript'
 */

#include <stdio.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>

//Quality assurement and debugging macros
#define QA(x) if(!(x)){fprintf(stderr, "Error encountered at line %d in file %s\n", __LINE__, __FILE__); exit(1);}
#define QAP(x, format, ...) if(!(x)){fprintf(stderr, "%s:%d\n", __FILE__, __LINE__); fprintf(stderr, format, __VA_ARGS__); exit(1);}

//global constants and variables
#define BUFSIZE 8196

int main(int argc, char *argv[])
{
	int sock, n;
	struct sockaddr_in server;
	struct hostent* hp;
	char buffer[BUFSIZE];
	unsigned long long written = 0;
	FILE* webscript;
		
	//check for arguments
	if( argc < 2 )
	{
		printf("Usage: %s [server] ([port] ([file]))\n", argv[0]);
		exit(1);
	}
	
	//open socket
	sock = socket(AF_INET, SOCK_STREAM, 0);
	QAP(sock >= 0, "Error opening socket.\n", NULL);
	
	//get hostname
	hp = gethostbyname(argv[1]);
	QAP(hp != NULL, "Unknown host.\n", NULL);
	
	//setup server
	bcopy((char *)hp->h_addr, (char *)&server.sin_addr, hp->h_length);
	server.sin_family = AF_INET;
	
	//select port
	if( argc >= 3 )
		server.sin_port = htons( (unsigned short)atoi(argv[2]) );
	else
		server.sin_port = htons(80);
	
	//connect to server
	if( connect(sock, (struct sockaddr *)&server, sizeof(server)) < 0 )
		QAP(0, "Error connection to server.\n", NULL);
	
	//connected successfully
	printf("Server: %s, Port: %s\n", argv[1], argv[2]);
	
	//open webscript
	webscript = fopen("webscript", "w");
	QAP(webscript > 0, "Error opening webscript for writing.\n", NULL);
	
	//build headers
	strcpy(buffer, "GET /");
	if( argc >= 4 )
		strncat(buffer, argv[3], BUFSIZE - strlen(buffer));
	strncat(buffer, " HTTP/1.1\r\n", BUFSIZE - strlen(buffer));
	
	//host header
	strncat(buffer, "Host: ", BUFSIZE - strlen(buffer));
	strncat(buffer, argv[1], BUFSIZE - strlen(buffer));
	strncat(buffer, "\r\n", BUFSIZE - strlen(buffer));
	
	//connection closed header, plus extra newline
	strncat(buffer, "Connection: close\r\n", BUFSIZE - strlen(buffer));
	strncat(buffer, "\r\n", BUFSIZE - strlen(buffer));
	
	//send headers
	n = write(sock, buffer, strlen(buffer));
	QAP(n == strlen(buffer), "Error sending headers.\n", NULL);
	
	do
	{
		//receive response
		strcpy(buffer, "\0");
		n = recv(sock, buffer, BUFSIZE-1, 0);
		QAP(n >= 0, "Error reading from socket.\n", NULL);
		buffer[n] = 0;
		
		//write to webscript
		fputs(buffer, webscript);
		written += n;
	}
	while( n > 0 );
	
	//print final info
	printf("Wrote %llu bytes.\nConnection closed.\n", written);
	
	close(sock);
	fclose(webscript);
	return 0;
}
